%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Explicit Runge-Kutta method for the IVP y'(t)=f(y,t), y(t0)=y0
% The method is defined by the Butcher array
%
% CALL  : [h,t,u]=exprk(fun,but,t0,y0,N,a)
%
% INPUT : fun  - name of the routine which determines the right hand side f(t,y) (Text-String)
%         but  - Butcher array   
%                     [  0       |    0                                   ]
%                     [alpha_{2} | beta_{2,1}                             ]  
%                 but=[  :       |    :       .                           ]  
%                     [alpha_{s} | beta_{s,1} ... beta_{s,s-1}            ]  
%                     [----------+----------------------------------------]
%                     [          | gamma_{1}  ...  gamma_{s-1}  gamma_{s} ]
%         t0   - initial point        t \in [t0,t0 + l]
%         y0   - initial value       y(t0)=y0
%         N    - number of steps
%         l    - lenght of the interval    t \in [t0,t0 + l]
%
% OUTPUT: h    - step size h=l/N
%         t    - mesh points t=[t0,t0+h,...,t0+(N-1)*h,t0+N*h=t0+l]
%         u    - approximate solution, i.e.,  u(i) = y(t(i)) 
%
function [h,t,u]=exprk(fun,but,t0,y0,N,l)
n=length(y0);
h=l/N;
t(1)=t0;
u(:,1)=y0;
[s,s1]=size(but);
if(s~=s1) error('Butcher array is not square'); end;
s=s-1;

for i=1:N
  % Compute k_{i}
  for j=1:s
    bk=zeros(n,1);
    for m=1:j-1
      if but(j,m+1)~=0
        bk=bk+but(j,m+1)*k(:,m);
      end;
    end;
    k(:,j)=feval(fun,t(i)+but(j,1)*h,u(:,i)+h*bk);
  end;
  %Compute next mesh point
  t(i+1)=t(i)+h;
  u(:,i+1)=u(:,i)+h*k*but(s+1,2:s+1)';
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

